/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#if !defined(AFX_TASKBAR_H__EBF1ABD2_1EA7_40AF_93CF_D4F2B4ECC8EE__INCLUDED_)
#define AFX_TASKBAR_H__EBF1ABD2_1EA7_40AF_93CF_D4F2B4ECC8EE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TaskbarButton.h"

#define LM_EXTERNALDRAW 9630
#define LM_EXTERNALDRAWNOTIFY 9640

class DropTarget;

class TaskbarTexture;
class TaskbarButton;
class ButtonTexture;
class ButtonIcon;
class ButtonFont;

//Alpha Transparency
#ifndef WS_EX_LAYERED
	#define WS_EX_LAYERED           0x00080000L
	#define LWA_COLORKEY            0x00000001L
	#define ULW_COLORKEY            0x00000001L
	#define LWA_ALPHA               0x00000002L
	#define ULW_ALPHA               0x00000002L
	#define ULW_OPAQUE              0x00000003L
#endif // WS_EX_LAYERED

#ifndef AC_SRC_ALPHA
	#define AC_SRC_ALPHA            0x00000001L
#endif

#ifndef WM_MOUSEWHEEL
	#define WM_MOUSEWHEEL 0x020A
#endif

typedef vector<TaskbarButton *>::iterator TaskbarButtonIterator;

class Taskbar  
{

public:
	Taskbar(const string &name);
	virtual ~Taskbar();

	DropTarget*	pDropTarget;

	void load(HINSTANCE hInstance);
	void reconfigure(bool refresh = false);

	const HWND &gethWnd() const { return hWnd; }
	const string &getName() const { return name; }
	const HWND &getTooltip() const { return ToolTips; }
	const HWND &getBox() const { return box; }
	bool isAlwaysOnTop() const { return alwaysOnTop; }
	bool isVisible() const { return visible; }
	bool isStartHidden() const { return startHidden; }
	bool isLayeredFlagSet() const { return layeredFlag; }

	void reposition(int type, int newx, int newy, int newwidth, int newheight, int steps = 0, int time = 0, bool repositionBy = false);

	void setBox(HWND newparent);
	void setAlwaysOnTop(bool setAlwaysOnTop);
	void setAlpha(int newAlphaTransparency = 255);

	void SwitchToButton( LPCSTR direction );

	void repaint();
	void updateRegion();

	void show();	
	void hide();

	bool useBigIcon;

	int displayButtonType;
	void CreateLayout();

	//For ReposButtons
	TaskbarButton *movecapture;

	//For Tooltips
	TaskbarButton *capture;

	//Access from TaskbarButton (FriendClass)
	bool noMinimizeOnClick;
	
	bool showIcon;
	bool showText;
	bool showToolTip;

	bool useGrouping;

	bool useHoverState;

	bool alphaMap;

	bool transparencyMode;

	ButtonTexture *buttontexture;
	ButtonIcon *buttonicon;
	ButtonFont *buttonfont;

	HWND lastActive;

	string taskbarSelect;

	vector<string> RemoveList;
	vector<string> CustomIconList;

	vector<string> AddMatchingList;
	vector<string> RemoveMatchingList;
		
	bool autoHidden;

private:

	bool hideIfEmpty;

	string afterClass;
	string afterTitle;

	bool createLayoutLocked;
	int addLock;
	int removeLock;

	bool VWMCurrentDesktopOnly;

	//For fixing Multiple Executions
	TaskbarButton *clickcapture;

	int backupExportedButtonCount;

	string name;

	bool alphaFade;
	int UpdateAlpha;
	int alphaFadeStep;
	int alphaFadeDelay;

	bool alphaInitialized;
	bool layeredFlag;
	int alphaTransparency;

	HWND hWnd;
	HINSTANCE hInstance;
	HWND box;
	HWND ToolTips;

	bool mouseInside;
	int focusOnEnter;

	bool actionsOnDownButton;

	vector<TaskbarButton *> taskbuttons;
	vector<TaskbarButton *> displayedtaskbuttons;

	HDC backgroundDC;
	HBITMAP backgroundBitmap;
	bool inCriticalProcess;

	TaskbarTexture *taskbartexture;

	int actualx;
	int actualy;
	int actualwidth;
	int actualheight;

	int leftBorder;
	int topBorder;
	int rightBorder;
	int bottomBorder;

	int maxTaskWidth;
	int maxTaskHeight;

	int layout;
	int direction;
	int wrapDirection;

	int lines_rows;
	int lines_rowsAdded;
	int wrapCount;
	int maxlines_rows;

	bool loadInBox;
	bool startHidden;
	bool alwaysOnTop;
	bool visible;

	//Actions
	int lButtonAction;
	int rButtonAction;
	int mButtonAction;

	bool minimizeAtAllCosts;

	string taskbarLButtonDown;
	string taskbarLButtonUp;
	string taskbarRButtonDown;
	string taskbarRButtonUp;
	string taskbarMButtonDown;
	string taskbarMButtonUp;
	string taskbarMWheelDown;
	string taskbarMWheelUp;
	string taskbarEnter;
	string taskbarLeave;
	string taskbarAdd;
	string taskbarRemove;
	string taskbarMove;
	string taskbarResize;

	string taskbarLineAdd;
	string taskbarLineRemove;

	//General Button Settings
	TaskbarButton *dropcapture;
	int dropTimeOut;
	int timerRefresh;
	int flashRefresh;
	int xSpacing;
	int ySpacing;

	int autoSort;

	void SortTasks(vector<TaskbarButton *> &taskbuttons);

	static bool CompareTasks(TaskbarButton *b1, TaskbarButton *b2)
	{
		string one;
		string two;

		if ( b1->taskbar->autoSort == 1 )
		{
			CString help;
			help = b1->classname.c_str();
			one = (string) help.Right(help.GetLength()-help.ReverseFind('\\')-1);
			one.append( b1->caption );
			help = b2->classname.c_str();
			two = (string) help.Right(help.GetLength()-help.ReverseFind('\\')-1);
			two.append( b2->caption );
		}
		else if ( b1->taskbar->autoSort == 2 )
		{
			one = GetTaskClass( b1->hButtonTask );
			one.append( b1->caption );
			two = GetTaskClass( b2->hButtonTask );
			two.append( b2->caption );
		}
		else if ( b1->taskbar->autoSort == 3 )
		{
			one = b1->caption;
			two = b2->caption;
		}
		
		if (lstrcmpi(one.c_str(), two.c_str()) < 0)
			return true;
		else
			return false;
	};

protected:

	virtual void onLButtonDown(int x, int y);
	virtual void onLButtonUp(int x, int y);
	virtual void onMButtonDown(int x, int y);
	virtual void onMButtonUp(int x, int y);
	virtual void onRButtonDown(int x, int y);
	virtual void onRButtonUp(int x, int y); 
	virtual void onWheelDown(int x, int y);
	virtual void onWheelUp(int x, int y);
	virtual void onMouseEnter();
	virtual void onMouseLeave();
	virtual void onMouseMove(int x, int y);
	virtual void onPaint(HDC hDC);
	virtual void onWindowPosChanging(WINDOWPOS *windowPos);
	virtual void onMove(int x, int y);
	virtual void onSize(int width, int height);
	virtual void onTimer(int timerID);

private:

	void onRedraw( HWND hApplication, bool fFlash );
	void onWindowActivated( HWND hApplication, BOOL fFullScreen);
	void onWindowCreated( HWND hApplication);
	void onWindowDestroyed( HWND hApplication);

	void updateHint(HWND hWnd, LPSTR caption, bool updatesize, bool remove = false);

private:
	BOOL onWindowMessage( UINT message, WPARAM wParam, LPARAM lParam, LRESULT &lResult );

private:   // functions

	void UpdateWindowTransparent(HWND hWnd, int factor);

	TaskbarButton *SearchForButtonByHandle( HWND search_hWnd );
	TaskbarButton *SearchForButtonByClass( HWND search_hWnd );
	TaskbarButton *ButtonAtPoint( int x, int y );

private:   // statics
	static BOOL WINAPI EnumWindowsProc( HWND hWnd, LPARAM lParam );

public:
	static LRESULT WINAPI windowProcedure( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );

};

#endif // !defined(AFX_TASKBAR_H__EBF1ABD2_1EA7_40AF_93CF_D4F2B4ECC8EE__INCLUDED_)
