/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "TaskbarTexture.h"

TaskbarTexture::TaskbarTexture()
{
	transparencyNeeded = false;

	useAlphaMap = false;

	Gradient.clear();

	memoryDC = 0;
	memoryBitmap = 0;

	useSolidColors = false;
}

TaskbarTexture::~TaskbarTexture()
{
	Gradient.clear();

	if(memoryBitmap)
	{
		memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
		DeleteObject(memoryBitmap);
	}
	if(memoryDC)
		DeleteDC(memoryDC);
}

void TaskbarTexture::parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor)
{
	int color;
	char main[32], light[32], dark[32];
	char *buffers[] = {main, light, dark};

	int numTokens = LCTokenize(cfg.c_str(), buffers, 3, NULL);

	if(numTokens == 1)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		lightcolor = GetSysColor(COLOR_3DLIGHT);
		darkcolor = GetSysColor(COLOR_3DSHADOW);
	}
	else if (numTokens == 2)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 3)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(dark, NULL, 16);
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
}

void TaskbarTexture::configure(const string &name, const bool &transparencyMode)
{
	NameValuePair modeConstants[] =
	{
		{ "stretch", MULTIBLT_STRETCH },
		{ "tile", MULTIBLT_TILE },
		{ "tile-horizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tile-vertical", MULTIBLT_TILEVERTICAL },
		{ "tilehorizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tilevertical", MULTIBLT_TILEVERTICAL },
		{ 0, 0 }
	};

	NameValuePair borderValues[] = 
	{
		{ ".none", 0 },
		{ "none", 0 },
		{ "raised", 1 },
		{ "sunken", 2 },
		{ "etched", 3 },
		{ "bump", 4 },
		{ 0, 0 }
	};

	if ( IsOS(OS_2KXP)  && !GetxRCBool(name, "UseSolidColors") && !GetxRCString(name, "Image", "").empty() )
		useAlphaMap = GetxRCBool(name, "AlphaMap");

	//Is useable in SolidColors and Image Mode
	borderMethod = GetxRCNamedValue(name, "BorderMethod", borderValues, 0);

	string imageFile = "";
	
	useSolidColors = GetxRCBool(name, "UseSolidColors");

	if ( useSolidColors )
	{
		imageFile = GetxRCLine(name, "SolidColors", "");
		if ( !imageFile.empty() )
			parseCFG(imageFile, backgroundColor, lightBevelColor, darkBevelColor);
		else
		{
			backgroundColor = GetSysColor(COLOR_BTNFACE);
			lightBevelColor = GetSysColor(COLOR_3DLIGHT);
			darkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		Gradient.insert(Gradient.end(), backgroundColor );

		if (backgroundColor == RGB(255,0,255) )
			transparencyNeeded = true; 

		bevelSize = GetxRCInt(name, "SolidBevelSize", 0, 0);

		verticalGradient = GetxRCBool(name, "SolidGradientVertical");

		imageFile = GetxRCLine(name, "SolidGradientColors", "");
	
		if ( !imageFile.empty() )
		{	
			int color;
			COLORREF insertColor;
			char token[4096];
			LPCSTR nextToken = imageFile.c_str();

			while (GetToken(nextToken, token, &nextToken, false))
			{
				color = strtol(token, NULL, 16);
				insertColor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		
				Gradient.insert(Gradient.end(), insertColor );
			}
		}
	}
	else
	{
		imageFile = GetxRCString(name, "Image", "");
		memoryBitmap = LoadLSImage(imageFile.c_str(), 0);

		if (memoryBitmap)
		{
			GetLSBitmapSize(memoryBitmap, &imageWidth, &imageHeight);

			memoryDC = CreateCompatibleDC(NULL);
			SelectObject(memoryDC, memoryBitmap);

			taskbar_saturation = GetxRCInt(name , "SaturationIntensity", 255, 0, 255);
			taskbar_hueIntensity = GetxRCInt(name , "HueIntensity", 0, 0, 255);
			taskbar_hueColor = GetxRCColor(name, "HueColor", RGB(0, 0, 0));

			leftEdge = GetxRCInt(name, "ImageLeftEdge", 0, 0);
			topEdge = GetxRCInt(name, "ImageTopEdge", 0, 0);
			rightEdge = GetxRCInt(name, "ImageRightEdge", 0, 0);
			bottomEdge = GetxRCInt(name, "ImageBottomEdge", 0, 0);

			mode = GetxRCNamedValue(name, "ImageMode", modeConstants, MULTIBLT_STRETCH);

			if (useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memoryBitmap);
			}
			else
			{
				paintEffects(memoryDC, imageWidth, imageHeight);
			}
		}
		else
			transparencyNeeded = true;
	}
}

void TaskbarTexture::apply(HDC hDC, int x, int y, int width, int height, bool trueTransparency)
{
	SetStretchBltMode(hDC, STRETCH_DELETESCANS);

	if (useSolidColors)
	{
		RECT r;
		r.left = x;
		r.top = y;
		r.right = r.left + width;
		r.bottom = r.top + height;

		// fill in the background
		if (bevelSize > 0 && (bevelSize*2 < width && bevelSize*2 < height))
		{
			::SetBkColor(hDC, darkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-bevelSize;
			r.bottom = r.bottom-bevelSize;
			::SetBkColor(hDC, lightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+bevelSize;
			r.top = r.top+bevelSize;

			if ( Gradient.size() > 1 )
			{
				solidGradientPaint(hDC, r, verticalGradient);
			}
			else
			{
				if (!trueTransparency && backgroundColor == RGB(255,0,255))
				{
					PaintDesktopEx(hDC, r.left, r.top, r.right-r.left, r.bottom-r.top, x, y, FALSE);
				}
				else
				{
					::SetBkColor(hDC, backgroundColor);
					::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
				}
			}
		}
		else
		{
			if ( Gradient.size() > 1 )
			{
				solidGradientPaint(hDC, r, verticalGradient);
			}
			else
			{
				if ( backgroundColor != RGB(255,0,255) )
				{
					::SetBkColor(hDC, backgroundColor);
					::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
				}
			}
		}
	}
	else
	{
		if (memoryBitmap == 0)
			return;

		if (trueTransparency)
		{
			MultiBlt(hDC, x, y, width, height,
						memoryDC, 0, 0, imageWidth, imageHeight,
						leftEdge, topEdge, rightEdge, bottomEdge,
						mode, SRCCOPY);
		}
		else if (!trueTransparency)
		{
			HDC bufferDC = CreateCompatibleDC(hDC);
			HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
			SelectObject(bufferDC, bufferBitmap);

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryDC, 0, 0, imageWidth, imageHeight,
					leftEdge, topEdge, rightEdge, bottomEdge,
					mode, SRCCOPY);
				
			TransparentBltLS(hDC, x, y, width, height,
					bufferDC, 0, 0,
					RGB(255, 0, 255));

			bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
		}
	}

	if ( !useAlphaMap )
	{
		RECT completeFillRect = {x, y, x+width, y+height};
		if (borderMethod == 1)
			DrawEdge(hDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
		else if (borderMethod == 2)
			DrawEdge(hDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
		else if (borderMethod == 3)
			DrawEdge(hDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
		else if (borderMethod == 4)
			DrawEdge(hDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
	}
}

void TaskbarTexture::paintEffects(HDC destHDC, int width, int height)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	int iy = 0;
	int ix = 0;
	COLORREF cl = 0;
	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	for(iy = 0; iy < height; iy++)
	{
		for(ix = 0; ix < width; ix++)
		{
			cl = GetPixel(destHDC, ix, iy);
			r = GetRValue(cl);
			g = GetGValue(cl);
			b = GetBValue(cl);
			
			if( !( r == 255 && g == 0 && b == 255 ) )
			{
				// Saturation effect
				if(taskbar_saturation != 255)
				{
					BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

					r = (BYTE)((r * taskbar_saturation + gray * (255 - taskbar_saturation) + 255) >> 8);
					g = (BYTE)((g * taskbar_saturation + gray * (255 - taskbar_saturation) + 255) >> 8);
					b = (BYTE)((b * taskbar_saturation + gray * (255 - taskbar_saturation) + 255) >> 8);
				}

				// Hue effect
				if(taskbar_hueIntensity != 0)
				{
					// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
					// test normal
					r = (BYTE)((r * (255 - taskbar_hueIntensity) + GetRValue(taskbar_hueColor) * taskbar_hueIntensity + 255) >> 8);
					g = (BYTE)((g * (255 - taskbar_hueIntensity) + GetGValue(taskbar_hueColor) * taskbar_hueIntensity + 255) >> 8);
					b = (BYTE)((b * (255 - taskbar_hueIntensity) + GetBValue(taskbar_hueColor) * taskbar_hueIntensity + 255) >> 8);
				}
			}
			else
				transparencyNeeded = true;

			SetPixelV(destHDC, ix, iy, RGB(r, g, b));
		}
	}
	//End Saturation / Hueing
}

void TaskbarTexture::solidGradientPaint(HDC destHDC, RECT r, bool vertical)
{
	int nWidth = r.right - r.left;
	int nHeight = r.bottom - r.top;

	RECT r2;
	double steps = 1;
	double cr, cg, cb, dr, dg, db;
	int gradSteps;

	int colorCount = Gradient.size();

	if (vertical)
		gradSteps = max(nHeight / (colorCount-1), 1);
	else
		gradSteps = max(nWidth / (colorCount-1), 1);

	int lastGradientPos;
	if (vertical)
		lastGradientPos = r.top;
	else
		lastGradientPos = r.left;

	for (ColorRefListIterator cit = Gradient.begin(); cit != Gradient.end(); ++cit )
	{
		if (cit == Gradient.end())
			break;

		COLORREF one = *cit;
		++cit;
		COLORREF two = *cit;
		--cit;
	
		// get current color
		cr = GetRValue( one );
		cg = GetGValue( one );
		cb = GetBValue( one );

		// Get delta color
		dr = (cr - GetRValue( two )) / gradSteps;
		dg = (cg - GetGValue( two )) / gradSteps;
		db = (cb - GetBValue( two )) / gradSteps;

		for (int i = 1; i <= gradSteps; i++)
		{
			cr -= dr;
			cg -= dg;
			cb -= db;

			if (vertical)
				SetRect(&r2, r.left, (int)(lastGradientPos+steps*(i - 1)), r.right, min((int)(lastGradientPos+steps*i), r.bottom));
			else
				SetRect(&r2, (int)(lastGradientPos+steps*(i - 1)), r.top, min((int)(lastGradientPos+steps*i), r.right), r.bottom);

			if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
			{
				::SetBkColor(destHDC, RGB((int)cr, (int)cg, (int)cb));
				::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
			}
		}
		if (vertical)
			lastGradientPos = r2.bottom;
		else
			lastGradientPos = r2.right;
	}
}

void TaskbarTexture::PrepareAlphaChannel(HBITMAP &bmp)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	BITMAP bm;
	GetObject(bmp, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	for (int i = 0; i < bm.bmHeight * bm.bmWidth; ++i)
	{
		//Magic Pink
		if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
			pPixel[3] = 0;

		//Prepare Text
		if (pPixel[3] == 0)
			pPixel[3] = 1;

		r = pPixel[0];
		g = pPixel[1];
		b = pPixel[2];

		//AlphaMap Colorizing
		//Saturation Effect
		if( taskbar_saturation != 255 )
		{
			BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);
			r = (BYTE)((r * taskbar_saturation + gray * (255 - taskbar_saturation) + 255) >> 8);
			g = (BYTE)((g * taskbar_saturation + gray * (255 - taskbar_saturation) + 255) >> 8);
			b = (BYTE)((b * taskbar_saturation + gray * (255 - taskbar_saturation) + 255) >> 8);
		}
		// Hue effect
		if( taskbar_hueIntensity != 0 )
		{
			// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
			// test normal
			b = (BYTE)((r * (255 - taskbar_hueIntensity) + GetRValue(taskbar_hueColor) * taskbar_hueIntensity + 255) >> 8);
			g = (BYTE)((g * (255 - taskbar_hueIntensity) + GetGValue(taskbar_hueColor) * taskbar_hueIntensity + 255) >> 8);
			r = (BYTE)((b * (255 - taskbar_hueIntensity) + GetBValue(taskbar_hueColor) * taskbar_hueIntensity + 255) >> 8);
		}
		
		//PreMultiply
		pPixel[0] = r * pPixel[3] / 255;
		pPixel[1] = g * pPixel[3] / 255;
		pPixel[2] = b * pPixel[3] / 255;

		pPixel+= 4;
	}
}